#!/usr/bin/python
from optparse import OptionParser
import re
import urlparse
from launchpadlib.launchpad import Launchpad
from subprocess import call
import sys
import os
import glob

qt_sources = ["qtsystems-opensource-src",
              "qtbase-opensource-src",
              "qtdeclarative-opensource-src",
              "qtmultimedia-opensource-src",
              "qtpim-opensource-src",
              "ubuntu-ui-toolkit",
              "qt3d-opensource-src",
              "qtfeedback-opensource-src",
              "qtlocation-opensource-src",
              "qtscript-opensource-src",
              "qtsensors-opensource-src",
              "qtsvg-opensource-src",
              "qttools-opensource-src",
              "qtwebsockets-opensource-src",
              "qtxmlpatterns-opensource-src",
              "qtquickcontrols-opensource-src",
              "qtconnectivity-opensource-src",
              "qtimageformats-opensource-src",
              "qtgraphicaleffects-opensource-src",
              "qtserialport-opensource-src",
              "qtenginio-opensource-src"
              ]

hack_set = set(['qt3d', 'qtpim', 'qtfeedback', 'qtsystems', 'ubuntu-sdk'])


def create_webroot_url_from_self_link(self_link):
    scheme, netloc, _, _, _ = urlparse.urlsplit(self_link)
    netloc = netloc.lstrip("api.")
    return u"%s://%s/" % (scheme, netloc)


def get_dsc(archive, package, series):
    re_version = re.compile(r"^\d+\:")
    x = archive.getPublishedSources(exact_match=True, source_name=package,
                                    distro_series=series)
    webroot = create_webroot_url_from_self_link(archive.self_link)
    version = x[0].source_package_version
    version = re_version.sub("", version, 1)
    if archive.owner.name == "ubuntu-archive":
        yield "%subuntu/+archive/primary/+files/%s_%s.dsc" \
            % (webroot, x[0].source_package_name, version)
    else:
        yield "%s/~ci-train-ppa-service/+archive/ubuntu/stable-phone-overlay/+files/%s_%s.dsc" \
            % (webroot, x[0].source_package_name, version)


def main():
    parser = OptionParser(usage="usage: %prog [options] source ...")
    parser.add_option(
        "-l", "--launchpad", dest="launchpad_instance", default="production")
    options, args = parser.parse_args()
    launchpad = Launchpad.login_anonymously('just testing', options.launchpad_instance)
    ubuntu = launchpad.distributions["ubuntu"]
    distro_archive = ubuntu.main_archive
    overlay_archive = launchpad.people["ci-train-ppa-service"].\
        getPPAByName(name="stable-phone-overlay")
    series = ubuntu.getSeries(name_or_version="vivid")

    for qt_source_package in qt_sources:
        re_version = re.compile(r"^\d+\:")

        #first try the overlay archive
        archive = overlay_archive
        x = overlay_archive.getPublishedSources(exact_match=True,
                                                source_name=qt_source_package,
                                                distro_series=series)
        if len(x) == 0:
            archive = distro_archive
            x = distro_archive.getPublishedSources(exact_match=True,
                                                   source_name=qt_source_package,
                                                   distro_series=series)
            if len(x) == 0:
                print("Could not find "+qt_source_package)
                sys.exit(1)
            else:
                print(qt_source_package+" found in Archive")
        else:
            print(qt_source_package+" found in overlay ppa")

        x = x[0].source_package_version

        generator = get_dsc(archive, qt_source_package, series)
        for i in generator:
            if qt_source_package != 'ubuntu-ui-toolkit':
                qt_module_name = re.sub(r'-opensource-src',
                                        r'',
                                        qt_source_package.rstrip())
            else:
                qt_module_name = re.sub(r'ubuntu-ui-toolkit',
                                        r'ubuntu-sdk',
                                        qt_source_package.rstrip())
            call(["dget", i, "--quiet", "--download-only"])
            for dsc_file in glob.glob('%s*dsc' % qt_source_package):
                call(["dpkg-source", "-x", dsc_file])
            for junk_files in glob.glob("*.?z"):
                os.remove(junk_files)
            for junk_files in glob.glob("*.dsc"):
                os.remove(junk_files)
            for fetched_module_directory in glob.glob('%s*' % qt_source_package):
                os.rename(fetched_module_directory, qt_module_name)
            if qt_module_name in hack_set:
                git_dir = qt_module_name + "/.git"
                if not os.path.exists(git_dir):
                    os.makedirs(git_dir)

main()
